# -*- coding: utf-8 -*-
"""
Created on Wed Aug  5 15:08:33 2020

@author: KDudley
"""

import re #a module that should come with python

#%% Simple example
first_example = 'simple text with the number 47 in it'
reg_pat = r'[0-9]+'
reg_first_result = re.findall(reg_pat, first_example)
print('\tSimple Result\n\t\t',reg_first_result)

print()


#%% Second example
second_example = "Find 259 that 111 word behind behind"

#With regex
reg_str = r'(?<!\d)(?:[0-46-9]+)(?!\d)\W*(\w+)'
reg_second_result = re.findall(reg_str, second_example)
print('\tYes Regex:\n\t\t', reg_second_result)

print()

#Without regex
split_str = second_example.split()
no_5 = [i.isnumeric() if '5' not in i else False for i in split_str]
idx_num = no_5.index(True)
the_word = split_str[idx_num + 1]
print('\tNot Regex:\n\t\t', the_word)


#%% Information
'''

see cheat sheet:
    https://www.debuggex.com/cheatsheet/regex/python
    or
    https://cheatography.com/davechild/cheat-sheets/regular-expressions/pdf/
    
    
Regex visual parsing tools online:
    https://pythex.org/
    or
    https://regexr.com/
    
    
Regex syntax breakdown:
    ()      - this is a group, most flavors of regex will try to return your results as groups if you have them
    (?<!)   - A Negative Lookbehind Assertion - This looks for items which must not precede the match
    \d      - shorthand for any digit alternatively you can write [0-9]
    (?:)    - No return group match - a group with the name ?: will make sure there is a match, but not return the match in the results
    [0-46-9] - matching group or range, anything in these brackets is matched as is (any combination of), the "-" in this case means between
                commonly you might see [a-zA-Z] to match any one letter between a to z or A to Z. the example here excludes 5.
                alternately to exclude 5 you can use an exclude group [^\D5] - \D is not a digit, and not 5
    +       - this means "one or more". with [0-9]+ I match any number 0-9, if the next character is also a match for this, match again, until it cannot
    (?!)    - Negative Lookahead assertion - this means that after a match is found, it must not be followed by a number
    \W      - not a word, generally anything not alphanumeric such as _,-, ,',",:,; etc...
    *       - "0 or more" of. Like + except that it is allowed to return no match at all and continue with the next match pattern
    \w      - a word, any alphanumeric character - generally 0-9 a-z A-Z
    +       - keep matching any alphanumeric character until there are no more to match.
    
    
Regex logic breakdown:
    (?<!\d)     - the match must not be preceded by a digit
    (?:[0-46-9]+) - match but not return group, match any number 0-4 or 6-9 [0-46-9] and keep matching + until unable
    (?!\d)      - the match must not be followed by a number
    \W*         - after the numeric match, IF there is any non-alphanumeric character match it, but do not return the match, keep going until unable
    (\w+)       - return group, match any alphanumeric character until unable

'''